from kivy.app import App
from kivy.uix.widget import Widget
from kivy.properties import ListProperty
from kivy.graphics import Color, Quad, Line,  Rectangle
from kivy.core.window import Window
import copy
import hvol5_2 as h5


class CubeGame(Widget):
    def __init__(self, **kwargs):
        super(CubeGame, self).__init__(**kwargs)

        self.hs = copy.deepcopy (self.hypersolids[0])
        self.draw4d()


    def on_touch_down(self, touch):
        self.clearhypersolid()
        self.kvcanvasupdate()
        self.hsindex = self.hsindex + 1
        if self.hsindex == self.totalhs:
            self.hsindex = 0
        self.hs = copy.deepcopy (self.hypersolids[self.hsindex])
        self.draw4d()
        
        self.hs1 = copy.deepcopy (self.hs)
        h5.uzyx01 (self.hs1)  # convert coordinates (u,z,y,x) from {-1, +1} in hs to {0, 1} in hs1
        self.clear3cube()
        u_1_flag = False
        for e in reversed (range (len(self.hs1))):
            posx = posy = 0
            cc = 2  # color for cube
            if self.hs1[e][3] == 1:
                posx = posx - 1
            if self.hs1[e][2] == 1:
                posy = posy - 1
            if self.hs1[e][1] == 1:
                posx = posx + 0.5
                posy = posy + 0.5
            if self.hs1[e][0] == 0:  # u = 0
                if u_1_flag == True:
                    if saved_u1_0 == self.hs1[e]:
                        cc = 3
                self.draw3cube(posx,  posy,  cc)
            else:  # u = 1 ; only one possible ; the first in reversed elementary cube list
                u_1_flag = True  # always followed with elementary cube with u = 0 and the same z,y,x
                saved_u1_0 = self.hs1[e]
                saved_u1_0[0] = 0
        return False

        
    def draw4d(self):
            self.hs1 = copy.deepcopy (self.hs)
            h5.uzyx01 (self.hs1)  # convert coordinates (u,z,y,x) from {-1, +1} in hs to {0, 1} in hs1
            for e in range (len(self.hs1)):
                self.fillcube [self.hs1[e][0]] [self.hs1[e][1]] [self.hs1[e][2]] [self.hs1[e][3]] ^= 2
            self.kvcanvasupdate()


    def draw3cube(self, posx, posy, cc):
        esz = min (self.height / 8, self.width / 16)
        p3dx = 11 * esz
        p3dy = 4 * esz
        dsz = 2 * esz
        t1x = p3dx + posx * dsz 
        t1y = p3dy + posy * dsz 
        t2x = t1x + dsz
        t2y = t1y
        t3x = t2x
        t3y = t2y + dsz
        t4x = t1x
        t4y = t3y
        t5x = t3x + esz
        t5y = t3y + esz
        t6x = t5x
        t6y = t5y - dsz
        t7x = t4x + esz
        t7y = t4y + esz
        color = self.fcolor[cc]
        with self.canvas:
            Color (1, 1, 1)
            Line (points = (t1x, t1y, t2x, t2y, t3x, t3y,  t4x, t4y,  t1x,  t1y) )
            Line (points = (t3x, t3y, t5x, t5y, t6x, t6y,  t2x, t2y) )
            Line (points = (t4x, t4y, t7x, t7y, t5x, t5y,  t3x, t3y) )
            Color (*color)
            Quad (points = (t1x + 1, t1y + 1, t2x - 1, t2y + 1, t3x - 1, t3y - 1,  t4x + 1, t4y - 1) )
            Quad (points = (t3x + 1, t3y - 1, t5x - 1, t5y - 1, t6x - 1, t6y + 1,  t2x + 1, t2y + 1) )
            Quad (points = (t4x + 1, t4y + 1, t7x + 1, t7y - 1, t5x - 1, t5y + 1,  t3x - 1, t3y + 1) )


    def clear3cube(self):
        esz = min (self.height / 8, self.width / 16)
        p3dx = 11 * esz
        p3dy = 4 * esz
        with self.canvas:
            Color (0, 0, 0)
            Rectangle (pos = (p3dx - 2 * esz - 2,  p3dy - 2 * esz - 2),  size = (esz * 8 + 4, esz * 8 + 4))


    def kvcanvasupdate(self):
        self.fillcube.append(4)  # .kv will see Property change
        del self.fillcube[2]


    def clearhypersolid(self):
        self.hs1 = copy.deepcopy (self.hs)
        h5.uzyx01 (self.hs1)
        for e in range (len(self.hs1)):
            self.fillcube [self.hs1[e][0]] [self.hs1[e][1]] [self.hs1[e][2]] [self.hs1[e][3]] ^= 2


    fillcube = ListProperty ([ [ [ [ 0 for l in range(2) ] for k in range(2) ] for j in range(2) ] for i in range(2) ])
    hs = []
    hs1 = []
    NoUserInputs = False
    
    fcolor = ListProperty ([ (0, 0, 0), (0, 0.125, 1), (1, 0, 0), (1, 1, 0) ])
    
    hypersolids = []

    hypersolids.append (h5.h1a0)
    hypersolids.append (h5.h2a1)
    hypersolids.append (h5.h3a2)
    hypersolids.append (h5.h4a2)
    hypersolids.append (h5.h4b3)
    hypersolids.append (h5.h4c3)
    hypersolids.append (h5.h5a3)
    hypersolids.append (h5.h5b3)
    hypersolids.append (h5.h5c4)
    hypersolids.append (h5.h5d4)
    hypersolids.append (h5.h5e4)
    hypersolids.append (h5.h5f4)

    totalhs = len (hypersolids)
    hsindex = 0
    

class Cube4DApp(App):
    def build (self):
        game4d = CubeGame()
        return game4d


if __name__ == '__main__':
    Cube4DApp().run()
